"use strict";
var update = require('immutability-helper');
var es6_promise_1 = require('es6-promise');
var array_1 = require('office-ui-fabric-react/lib/utilities/array');
var ADD_ITEMS_DELAY = 500;
/**
 * DataProvider the class used to maintain the client side data.
 *
 * It interact with data provider to sync data items with sharepoint list.
 * It also maintain the loading state to tell React Component once the action is
 * started of finished.
 */
var DataProvider = (function () {
    function DataProvider() {
        this._items = [
            {
                id: '61b59681-2a82-4a51-b221-8c35e333ae89',
                title: 'Finish Sample Todo web part before dev kitchen',
                isComplete: false
            },
            {
                id: '94a844ae-0c6a-4820-8042-dbc386bdf930',
                title: 'Finish All the work in Todo web part before dev kitchen',
                isComplete: false
            },
            {
                id: '5fa55618-90f9-4b5f-b12d-60c9fb1fc7f0',
                title: 'SharePoint API investigation for Todo web part',
                isComplete: true
            },
            {
                id: '2ae54c74-1395-4a49-8dd2-4857efdd0e5e',
                title: 'Bug fixing of Pivot Control',
                isComplete: true
            }
        ];
        this._isLoading = false;
        this._listeners = [];
        this.createItem = this.createItem.bind(this);
        this.deleteItem = this.deleteItem.bind(this);
        this.toggleComplete = this.toggleComplete.bind(this);
    }
    Object.defineProperty(DataProvider.prototype, "items", {
        /**
         * The items store in the local. It only contains the data recently fetched from server.
         */
        get: function () { return this._items; },
        set: function (value) {
            this._items = value;
            this._emitChange();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DataProvider.prototype, "isLoading", {
        /**
         * Whether there is unfinished server request currently.
         */
        get: function () { return this._isLoading; },
        set: function (value) {
            this._isLoading = value;
            this._emitChange();
        },
        enumerable: true,
        configurable: true
    });
    /**
     * Create a new item and add it to the list through data provider.
     */
    DataProvider.prototype.createItem = function (title) {
        var _this = this;
        this.isLoading = true;
        return new es6_promise_1.Promise(function (resolve) {
            var newItem = {
                id: _this._generateGuid(),
                title: title,
                isComplete: false
            };
            setTimeout(function () {
                _this.items = _this.items.concat(newItem);
                _this.isLoading = false;
                resolve(_this.items);
            }, ADD_ITEMS_DELAY);
        });
    };
    /**
     * Delete a item from the list through data provider.
     */
    DataProvider.prototype.deleteItem = function (delItem) {
        var _this = this;
        return new es6_promise_1.Promise(function (resolve) {
            _this.items =
                _this.items.filter(function (item) { return item.id !== delItem.id; });
            resolve(_this.items);
        });
    };
    /**
     * Toggle the complete state of an item by.
     *
     * Will call updateItem function to update complete state of this item.
     */
    DataProvider.prototype.toggleComplete = function (item) {
        var _this = this;
        // Create a new Item in which the PercentComplete value has been changed.
        var newItem = update(item, {
            isComplete: { $set: item.isComplete === true ? false : true }
        });
        return new es6_promise_1.Promise(function (resolve, reject) {
            var index = array_1.findIndex(_this.items, function (item) { return item.id === newItem.id; });
            if (index !== -1) {
                _this.items[index] = newItem;
                _this.items = _this.items.slice(0);
                resolve(_this.items);
            }
            else {
                reject(new Error("Item to update doesn't exist."));
            }
        });
    };
    /**
     * Add listener to the provider.
     *
     * Once the store has a change of state and emit that change, the listeners will be called.
     */
    DataProvider.prototype.addListener = function (listener) {
        this._listeners.push(listener);
    };
    /**
     * Remove the registered listener.
     *
     * You must remove the listener registered by addListener method when you are not going to use it anymore.
     */
    DataProvider.prototype.removeListener = function (listener) {
        var listenerIdx = this._listeners.indexOf(listener);
        if (listenerIdx > -1) {
            this._listeners.splice(listenerIdx, 1);
        }
    };
    /**
     * Emit the changes in the store to all listeners.
     */
    DataProvider.prototype._emitChange = function () {
        this._listeners.forEach(function (listener) { return listener(); });
    };
    DataProvider.prototype._generateGuid = function () {
        return 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function (c) {
            var r = Math.random() * 16 | 0, v = c == 'x' ? r : (r & 0x3 | 0x8);
            return v.toString(16);
        });
    };
    return DataProvider;
}());
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = DataProvider;

//# sourceMappingURL=DataProvider.js.map
